@@ -84,6 +84,11 @@ It supports various compressed files(gzip, bzip2, zstd, lz4, and xz).
 		if completion != "" {
 			return Completion(cmd, completion)
 		}
+		// Set the caption from the environment variable.
+		if config.General.Caption == "" {
+			config.General.Caption = viper.GetString("CAPTION")
+		}
+
 		// Actually tabs when "\t" is specified as an option.
 		if config.General.ColumnDelimiter == "\\t" {
 			config.General.ColumnDelimiter = "\t"
@@ -243,6 +248,8 @@ func flagUsage(f *pflag.FlagSet) string {
 
 	lines := make([]string, 0)
 
+	lines = append(lines, "Short   Long\x00Purpose")
+
 	maxlen := 0
 	f.VisitAll(func(flag *pflag.Flag) {
 		if flag.Hidden {
@@ -402,7 +409,7 @@ func init() {
 	rootCmd.PersistentFlags().BoolP("section-header", "", false, "enable section-delimiter line as Header")
 	_ = viper.BindPFlag("general.SectionHeader", rootCmd.PersistentFlags().Lookup("section-header"))
 
-	rootCmd.PersistentFlags().IntP("section-header-num", "", 1, "number of header lines")
+	rootCmd.PersistentFlags().IntP("section-header-num", "", 1, "number of section header lines")
 	_ = viper.BindPFlag("general.SectionHeaderNum", rootCmd.PersistentFlags().Lookup("section-header-num"))
 
 	rootCmd.PersistentFlags().BoolP("follow-mode", "f", false, "monitor file and display new content as it is written")
@@ -432,9 +439,12 @@ func init() {
 	rootCmd.PersistentFlags().StringP("caption", "", "", "custom caption")
 	_ = viper.BindPFlag("general.Caption", rootCmd.PersistentFlags().Lookup("caption"))
 
+	rootCmd.PersistentFlags().BoolP("hide-other-section", "", false, "hide other section")
+	_ = viper.BindPFlag("general.HideOtherSection", rootCmd.PersistentFlags().Lookup("hide-other-section"))
+
 	// Config
-	rootCmd.PersistentFlags().BoolP("disable-mouse", "", false, "disable mouse support")
-	_ = viper.BindPFlag("DisableMouse", rootCmd.PersistentFlags().Lookup("disable-mouse"))
+	rootCmd.PersistentFlags().BoolP("quit-if-one-screen", "F", false, "quit if the output fits on one screen")
+	_ = viper.BindPFlag("QuitSmall", rootCmd.PersistentFlags().Lookup("quit-if-one-screen"))
 
 	rootCmd.PersistentFlags().BoolP("exit-write", "X", false, "output the current screen when exiting")
 	_ = viper.BindPFlag("IsWriteOriginal", rootCmd.PersistentFlags().Lookup("exit-write"))
@@ -445,9 +455,6 @@ func init() {
 	rootCmd.PersistentFlags().IntP("exit-write-after", "a", 0, "number after the current lines when exiting")
 	_ = viper.BindPFlag("AfterWriteOriginal", rootCmd.PersistentFlags().Lookup("exit-write-after"))
 
-	rootCmd.PersistentFlags().BoolP("quit-if-one-screen", "F", false, "quit if the output fits on one screen")
-	_ = viper.BindPFlag("QuitSmall", rootCmd.PersistentFlags().Lookup("quit-if-one-screen"))
-
 	rootCmd.PersistentFlags().BoolP("case-sensitive", "i", false, "case-sensitive in search")
 	_ = viper.BindPFlag("CaseSensitive", rootCmd.PersistentFlags().Lookup("case-sensitive"))
 
@@ -460,18 +467,21 @@ func init() {
 	rootCmd.PersistentFlags().BoolP("incsearch", "", true, "incremental search")
 	_ = viper.BindPFlag("Incsearch", rootCmd.PersistentFlags().Lookup("incsearch"))
 
-	rootCmd.PersistentFlags().StringP("view-mode", "", "", "apply predefined settings for a specific mode")
-	_ = viper.BindPFlag("ViewMode", rootCmd.PersistentFlags().Lookup("view-mode"))
-
 	rootCmd.PersistentFlags().IntP("memory-limit", "", -1, "number of chunks to limit in memory")
 	_ = viper.BindPFlag("MemoryLimit", rootCmd.PersistentFlags().Lookup("memory-limit"))
 
 	rootCmd.PersistentFlags().IntP("memory-limit-file", "", 100, "number of chunks to limit in memory for the file")
 	_ = viper.BindPFlag("MemoryLimitFile", rootCmd.PersistentFlags().Lookup("memory-limit-file"))
 
+	rootCmd.PersistentFlags().BoolP("disable-mouse", "", false, "disable mouse support")
+	_ = viper.BindPFlag("DisableMouse", rootCmd.PersistentFlags().Lookup("disable-mouse"))
+
 	rootCmd.PersistentFlags().BoolP("disable-column-cycle", "", false, "disable column cycling")
 	_ = viper.BindPFlag("DisableColumnCycle", rootCmd.PersistentFlags().Lookup("disable-column-cycle"))
 
+	rootCmd.PersistentFlags().StringP("view-mode", "", "", "apply predefined settings for a specific mode")
+	_ = viper.BindPFlag("ViewMode", rootCmd.PersistentFlags().Lookup("view-mode"))
+
 	rootCmd.PersistentFlags().BoolP("debug", "", false, "debug mode")
 	_ = viper.BindPFlag("Debug", rootCmd.PersistentFlags().Lookup("debug"))
 }
