@@ -1,6 +1,7 @@
 package oviewer
 
 import (
+	"context"
 	"fmt"
 	"log"
 	"math"
@@ -14,7 +15,7 @@ import (
 )
 
 // toggleWrapMode toggles wrapMode each time it is called.
-func (root *Root) toggleWrapMode() {
+func (root *Root) toggleWrapMode(context.Context) {
 	m := root.Doc
 	m.WrapMode = !m.WrapMode
 
@@ -32,7 +33,7 @@ func (root *Root) toggleWrapMode() {
 }
 
 // toggleColumnMode toggles ColumnMode each time it is called.
-func (root *Root) toggleColumnMode() {
+func (root *Root) toggleColumnMode(context.Context) {
 	root.Doc.ColumnMode = !root.Doc.ColumnMode
 
 	if root.Doc.ColumnMode {
@@ -42,7 +43,7 @@ func (root *Root) toggleColumnMode() {
 }
 
 // toggleColumnWidth toggles ColumnWidth each time it is called.
-func (root *Root) toggleColumnWidth() {
+func (root *Root) toggleColumnWidth(context.Context) {
 	if root.Doc.ColumnWidth {
 		root.Doc.ColumnWidth = false
 		root.Doc.ColumnMode = false
@@ -55,37 +56,37 @@ func (root *Root) toggleColumnWidth() {
 }
 
 // toggleAlternateRows toggles the AlternateRows each time it is called.
-func (root *Root) toggleAlternateRows() {
+func (root *Root) toggleAlternateRows(context.Context) {
 	root.Doc.AlternateRows = !root.Doc.AlternateRows
 	root.setMessagef("Set AlternateRows %t", root.Doc.AlternateRows)
 }
 
 // toggleLineNumMode toggles LineNumMode every time it is called.
-func (root *Root) toggleLineNumMode() {
+func (root *Root) toggleLineNumMode(ctx context.Context) {
 	root.Doc.LineNumMode = !root.Doc.LineNumMode
-	root.ViewSync()
+	root.ViewSync(ctx)
 	root.setMessagef("Set LineNumMode %t", root.Doc.LineNumMode)
 }
 
 // togglePlain toggles plain mode.
-func (root *Root) togglePlain() {
+func (root *Root) togglePlain(context.Context) {
 	root.Doc.PlainMode = !root.Doc.PlainMode
 	root.setMessagef("Set PlainMode %t", root.Doc.PlainMode)
 }
 
 // togglePlain toggles column rainbow mode.
-func (root *Root) toggleRainbow() {
+func (root *Root) toggleRainbow(context.Context) {
 	root.Doc.ColumnRainbow = !root.Doc.ColumnRainbow
 	root.setMessagef("Set Column Rainbow Mode %t", root.Doc.ColumnRainbow)
 }
 
 // toggleFollowMode toggles follow mode.
-func (root *Root) toggleFollowMode() {
+func (root *Root) toggleFollowMode(context.Context) {
 	root.Doc.FollowMode = !root.Doc.FollowMode
 }
 
 // toggleFollowAll toggles follow all mode.
-func (root *Root) toggleFollowAll() {
+func (root *Root) toggleFollowAll(context.Context) {
 	root.General.FollowAll = !root.General.FollowAll
 	root.mu.Lock()
 	for _, doc := range root.DocList {
@@ -95,12 +96,31 @@ func (root *Root) toggleFollowAll() {
 }
 
 // toggleFollowSection toggles follow section mode.
-func (root *Root) toggleFollowSection() {
+func (root *Root) toggleFollowSection(context.Context) {
 	root.Doc.FollowSection = !root.Doc.FollowSection
 }
 
+// toggleHideOtherSection toggles hide other section mode.
+func (root *Root) toggleHideOtherSection(context.Context) {
+	root.Doc.HideOtherSection = !root.Doc.HideOtherSection
+	root.setMessagef("Set HideOtherSection %t", root.Doc.HideOtherSection)
+}
+
+// toggleMouse toggles mouse control.
+// When disabled, the mouse is controlled on the terminal side.
+func (root *Root) toggleMouse(context.Context) {
+	root.Config.DisableMouse = !root.Config.DisableMouse
+	if root.Config.DisableMouse {
+		root.Screen.DisableMouse()
+		root.setMessage("Disable Mouse")
+	} else {
+		root.Screen.EnableMouse(MouseFlags)
+		root.setMessage("Enable Mouse")
+	}
+}
+
 // closeFile close the file.
-func (root *Root) closeFile() {
+func (root *Root) closeFile(context.Context) {
 	if root.Doc.documentType == DocHelp || root.Doc.documentType == DocLog {
 		return
 	}
@@ -129,7 +149,7 @@ func (root *Root) reload(m *Document) {
 }
 
 // toggleWatch toggles watch mode.
-func (root *Root) toggleWatch() {
+func (root *Root) toggleWatch(context.Context) {
 	if root.Doc.WatchMode {
 		root.Doc.unWatchMode()
 	} else {
@@ -170,25 +190,27 @@ func (root *Root) watchControl() {
 
 // searchGo will go to the line with the matching term after searching.
 // Jump by section if JumpTargetSection is true.
-func (root *Root) searchGo(lN int) {
+func (root *Root) searchGo(ctx context.Context, lN int) {
 	root.resetSelect()
+	root.Doc.lastSearchLN = lN
 	x := root.searchXPos(lN)
 	if root.Doc.jumpTargetSection {
-		root.Doc.searchGoSection(lN, x)
+		root.Doc.searchGoSection(ctx, lN, x)
 		return
 	}
+	root.debugMessage(fmt.Sprintf("searchGo:%d->%d", root.Doc.topLN, lN))
 	root.Doc.searchGoTo(lN, x)
 }
 
 // goLine will move to the specified line.
 // decimal number > line number
-// 10 -> line 10
+// 10 -> line 10.
 // decimal number + "." + decimal number > line number + number of wrapping lines
-// 10.5 -> line 10 + 5 wrapping lines
+// 10.5 -> line 10 + 5 wrapping lines.
 // "." + decimal is a percentage position
-// .5 -> 50% of the way down the file
+// .5 -> 50% of the way down the file.
 // decimal + "%" is a percentage position
-// 50% -> 50% of the way down the file
+// 50% -> 50% of the way down the file.
 func (root *Root) goLine(input string) {
 	if len(input) == 0 {
 		return
@@ -203,6 +225,7 @@ func (root *Root) goLine(input string) {
 			return
 		}
 		lN = root.Doc.moveLine(lN - 1)
+		root.Doc.showGotoF = true
 		root.setMessagef("Moved to line %d", lN+1)
 		return
 	}
@@ -224,13 +247,13 @@ func (root *Root) goLine(input string) {
 }
 
 // goLineNumber moves to the specified line number.
-func (root *Root) goLineNumber(ln int) {
-	ln = root.Doc.moveLine(ln - root.Doc.firstLine())
-	root.setMessagef("Moved to line %d", ln+1)
+func (root *Root) goLineNumber(lN int) {
+	lN = root.Doc.moveLine(lN - root.Doc.firstLine())
+	root.setMessagef("Moved to line %d", lN+1)
 }
 
 // markNext moves to the next mark.
-func (root *Root) markNext() {
+func (root *Root) markNext(context.Context) {
 	if len(root.Doc.marked) == 0 {
 		return
 	}
@@ -244,7 +267,7 @@ func (root *Root) markNext() {
 }
 
 // markPrev moves to the previous mark.
-func (root *Root) markPrev() {
+func (root *Root) markPrev(context.Context) {
 	if len(root.Doc.marked) == 0 {
 		return
 	}
@@ -258,27 +281,27 @@ func (root *Root) markPrev() {
 }
 
 // addMark marks the current line number.
-func (root *Root) addMark() {
-	c := min(root.Doc.topLN+root.Doc.firstLine(), root.Doc.BufEndNum())
-	root.Doc.marked = remove(root.Doc.marked, c)
-	root.Doc.marked = append(root.Doc.marked, c)
-	root.setMessagef("Marked to line %d", c-root.Doc.firstLine()+1)
+func (root *Root) addMark(context.Context) {
+	lN := min(root.Doc.topLN+root.Doc.firstLine(), root.Doc.BufEndNum())
+	root.Doc.marked = remove(root.Doc.marked, lN)
+	root.Doc.marked = append(root.Doc.marked, lN)
+	root.setMessagef("Marked to line %d", lN-root.Doc.firstLine()+1)
 }
 
 // removeMark removes the current line number from the mark.
-func (root *Root) removeMark() {
-	c := root.Doc.topLN + root.Doc.firstLine()
-	marked := remove(root.Doc.marked, c)
+func (root *Root) removeMark(context.Context) {
+	lN := root.Doc.topLN + root.Doc.firstLine()
+	marked := remove(root.Doc.marked, lN)
 	if len(root.Doc.marked) == len(marked) {
-		root.setMessagef("Not marked line %d", c-root.Doc.firstLine()+1)
+		root.setMessagef("Not marked line %d", lN-root.Doc.firstLine()+1)
 		return
 	}
 	root.Doc.marked = marked
-	root.setMessagef("Remove the mark at line %d", c-root.Doc.firstLine()+1)
+	root.setMessagef("Remove the mark at line %d", lN-root.Doc.firstLine()+1)
 }
 
 // removeAllMark removes all marks.
-func (root *Root) removeAllMark() {
+func (root *Root) removeAllMark(context.Context) {
 	root.Doc.marked = nil
 	root.Doc.markedPoint = 0
 	root.setMessage("Remove all marks")
@@ -286,13 +309,9 @@ func (root *Root) removeAllMark() {
 
 // setHeader sets the number of lines in the header.
 func (root *Root) setHeader(input string) {
-	num, err := strconv.Atoi(input)
+	num, err := specifyOnScreen(input, root.scr.vHeight-1)
 	if err != nil {
-		root.setMessagef("Set header: %s", ErrInvalidNumber.Error())
-		return
-	}
-	if num < 0 || num > root.scr.vHeight-1 {
-		root.setMessagef("Set header %d: %s", num, ErrOutOfRange.Error())
+		root.setMessagef("Set header lines: %s", err.Error())
 		return
 	}
 	if root.Doc.Header == num {
@@ -306,13 +325,9 @@ func (root *Root) setHeader(input string) {
 
 // setSkipLines sets the number of lines to skip.
 func (root *Root) setSkipLines(input string) {
-	num, err := strconv.Atoi(input)
+	num, err := specifyOnScreen(input, root.scr.vHeight-1)
 	if err != nil {
-		root.setMessagef("Set skip line: %s", ErrInvalidNumber.Error())
-		return
-	}
-	if num < 0 {
-		root.setMessagef("Set skip line: %s", ErrOutOfRange.Error())
+		root.setMessagef("Set skip lines: %s", err.Error())
 		return
 	}
 	if root.Doc.SkipLines == num {
@@ -323,14 +338,11 @@ func (root *Root) setSkipLines(input string) {
 	root.setMessagef("Set skip lines %d", num)
 }
 
+// setSectionNum sets the number of section headers.
 func (root *Root) setSectionNum(input string) {
-	num, err := strconv.Atoi(input)
+	num, err := specifyOnScreen(input, root.scr.vHeight-1)
 	if err != nil {
-		root.setMessagef("Set section header num: %s", ErrInvalidNumber.Error())
-		return
-	}
-	if num < 0 {
-		root.setMessagef("Set section header num: %s", ErrOutOfRange.Error())
+		root.setMessagef("Set section header num: %s", err.Error())
 		return
 	}
 	if root.Doc.SectionHeaderNum == num {
@@ -341,9 +353,20 @@ func (root *Root) setSectionNum(input string) {
 	root.setMessagef("Set section header num %d", num)
 }
 
+func specifyOnScreen(input string, max int) (int, error) {
+	num, err := strconv.Atoi(input)
+	if err != nil {
+		return 0, ErrInvalidNumber
+	}
+	if num < 0 || num > max {
+		return 0, ErrOutOfRange
+	}
+	return num, nil
+}
+
 // suspend suspends the current screen display and runs the shell.
 // It will return when you exit the shell.
-func (root *Root) suspend() {
+func (root *Root) suspend(context.Context) {
 	log.Println("Suspend")
 	if err := root.Screen.Suspend(); err != nil {
 		log.Println(err)
@@ -363,7 +386,7 @@ func (root *Root) suspend() {
 	c.Stdout = os.Stdout
 	c.Stderr = os.Stderr
 	if err := c.Run(); err != nil {
-		root.setMessageLog(err.Error())
+		fmt.Fprintf(os.Stderr, "failed to run shell: %s\n", err)
 	}
 	fmt.Println("resume ov")
 	if err := root.Screen.Resume(); err != nil {
@@ -372,22 +395,9 @@ func (root *Root) suspend() {
 	log.Println("Resume")
 }
 
-// toggleMouse toggles mouse control.
-// When disabled, the mouse is controlled on the terminal side.
-func (root *Root) toggleMouse() {
-	root.Config.DisableMouse = !root.Config.DisableMouse
-	if root.Config.DisableMouse {
-		root.Screen.DisableMouse()
-		root.setMessage("Disable Mouse")
-	} else {
-		root.Screen.EnableMouse(MouseFlags)
-		root.setMessage("Enable Mouse")
-	}
-}
-
 // setViewMode switches to the preset display mode.
 // Set header lines and columnMode together.
-func (root *Root) setViewMode(modeName string) {
+func (root *Root) setViewMode(ctx context.Context, modeName string) {
 	c, ok := root.Config.Mode[modeName]
 	if !ok {
 		if modeName != "general" {
@@ -403,7 +413,7 @@ func (root *Root) setViewMode(modeName string) {
 	}
 	root.Doc.regexpCompile()
 	root.Doc.ClearCache()
-	root.ViewSync()
+	root.ViewSync(ctx)
 	root.setMessagef("Set mode %s", modeName)
 }
 
@@ -452,7 +462,7 @@ func (root *Root) setWatchInterval(input string) {
 
 // setWriteBA sets the number before and after the line
 // to be written at the end.
-func (root *Root) setWriteBA(input string) {
+func (root *Root) setWriteBA(ctx context.Context, input string) {
 	before, after, err := rangeBA(input)
 	if err != nil {
 		root.setMessage(ErrInvalidNumber.Error())
@@ -462,7 +472,7 @@ func (root *Root) setWriteBA(input string) {
 	root.AfterWriteOriginal = after
 	root.debugMessage(fmt.Sprintf("Before:After:%d:%d", root.BeforeWriteOriginal, root.AfterWriteOriginal))
 	root.IsWriteOriginal = true
-	root.Quit()
+	root.Quit(ctx)
 }
 
 // rangeBA returns the before after number from a string.
@@ -505,7 +515,10 @@ func (root *Root) setSectionStart(input string) {
 		root.setMessagef("Set section start position: %s", ErrInvalidNumber.Error())
 		return
 	}
-
+	if num < -root.scr.vHeight || num > root.scr.vHeight-1 {
+		root.setMessagef("Set section start position: %s", ErrOutOfRange.Error())
+		return
+	}
 	root.Doc.SectionStartPosition = num
 	root.setMessagef("Set section start position %s", input)
 }
@@ -526,7 +539,7 @@ func (root *Root) setMultiColor(input string) {
 
 // setJumpTarget sets the position of the search result.
 func (root *Root) setJumpTarget(input string) {
-	num, section := jumpPosition(root.scr.vHeight, input)
+	num, section := jumpPosition(input, root.scr.vHeight)
 	root.Doc.jumpTargetSection = section
 	if root.Doc.jumpTargetSection {
 		root.setMessagef("Set JumpTarget section start")
@@ -536,21 +549,21 @@ func (root *Root) setJumpTarget(input string) {
 		root.setMessagef("Set JumpTarget %d: %s", num, ErrOutOfRange.Error())
 		return
 	}
-	if root.Doc.jumpTargetNum == num {
+	if root.Doc.jumpTargetHeight == num {
 		return
 	}
 	root.Doc.JumpTarget = input
-	root.Doc.jumpTargetNum = num
+	root.Doc.jumpTargetHeight = num
 	root.setMessagef("Set JumpTarget %d", num)
 }
 
 // resize is a wrapper function that calls viewSync.
-func (root *Root) resize() {
-	root.ViewSync()
+func (root *Root) resize(ctx context.Context) {
+	root.ViewSync(ctx)
 }
 
 // jumpPosition determines the position of the jump.
-func jumpPosition(height int, str string) (int, bool) {
+func jumpPosition(str string, height int) (int, bool) {
 	if len(str) == 0 {
 		return 0, false
 	}
@@ -591,7 +604,7 @@ func calculatePosition(length int, str string) float64 {
 		p = i / 100
 	}
 
-	if p != 0 {
+	if p > 0 {
 		return float64(length) * p
 	}
 
@@ -602,44 +615,23 @@ func calculatePosition(length int, str string) float64 {
 	return num
 }
 
-// ViewSync redraws the whole thing.
-func (root *Root) ViewSync() {
-	root.resetSelect()
-	root.prepareStartX()
-	root.prepareView()
-	root.Screen.Sync()
-	root.Doc.jumpTargetNum, root.Doc.jumpTargetSection = jumpPosition(root.scr.vHeight, root.Doc.JumpTarget)
-}
-
 // TailSync move to tail and sync.
-func (root *Root) TailSync() {
+func (root *Root) TailSync(ctx context.Context) {
 	root.Doc.moveBottom()
-	root.ViewSync()
+	root.ViewSync(ctx)
 }
 
 // tailSection moves to the last section
 // and adjusts to its original position.
-func (root *Root) tailSection() {
+func (root *Root) tailSection(ctx context.Context) {
 	moved := root.Doc.topLN - root.Doc.lastSectionPosNum
-	root.Doc.moveLastSection()
+	root.Doc.moveLastSection(ctx)
 	if moved > 0 && (root.Doc.topLN+moved) < root.Doc.BufEndNum() {
 		root.Doc.moveLine(root.Doc.topLN + moved)
 	}
 	root.Doc.lastSectionPosNum = root.Doc.topLN
 }
 
-// prepareStartX prepares startX.
-func (root *Root) prepareStartX() {
-	root.scr.startX = 0
-	if root.Doc.LineNumMode {
-		if root.Doc.parent != nil {
-			root.scr.startX = len(fmt.Sprintf("%d", root.Doc.parent.BufEndNum())) + 1
-			return
-		}
-		root.scr.startX = len(fmt.Sprintf("%d", root.Doc.BufEndNum())) + 1
-	}
-}
-
 // updateEndNum updates the last line number.
 func (root *Root) updateEndNum() {
 	root.debugMessage(fmt.Sprintf("Update EndNum:%d", root.Doc.BufEndNum()))
@@ -649,9 +641,9 @@ func (root *Root) updateEndNum() {
 }
 
 // follow updates the document in follow mode.
-func (root *Root) follow() {
+func (root *Root) follow(ctx context.Context) {
 	if root.General.FollowAll {
-		root.followAll()
+		root.followAll(ctx)
 	}
 	num := root.Doc.BufEndNum()
 	if root.Doc.latestNum == num {
@@ -660,16 +652,16 @@ func (root *Root) follow() {
 
 	root.skipDraw = false
 	if root.Doc.FollowSection {
-		root.tailSection()
+		root.tailSection(ctx)
 	} else {
-		root.TailSync()
+		root.TailSync(ctx)
 	}
 	root.Doc.latestNum = num
 }
 
 // followAll monitors and switches all document updates
 // in follow all mode.
-func (root *Root) followAll() {
+func (root *Root) followAll(ctx context.Context) {
 	if root.Doc.documentType != DocNormal {
 		return
 	}
@@ -684,18 +676,31 @@ func (root *Root) followAll() {
 	root.mu.RUnlock()
 
 	if root.CurrentDoc != current {
-		root.switchDocument(current)
+		root.switchDocument(ctx, current)
 	}
 }
 
 // Cancel follow mode and follow all mode.
-func (root *Root) Cancel() {
+func (root *Root) Cancel(context.Context) {
 	root.General.FollowAll = false
 	root.Doc.FollowMode = false
 }
 
 // WriteQuit sets the write flag and executes a quit event.
-func (root *Root) WriteQuit() {
+func (root *Root) WriteQuit(ctx context.Context) {
 	root.IsWriteOriginal = true
-	root.Quit()
+	if !root.Doc.HideOtherSection {
+		root.Quit(ctx)
+		return
+	}
+
+	for lN := 0; lN < root.Doc.BufEndNum(); lN++ {
+		line := root.scr.lines[lN]
+		if line.section > 1 {
+			root.AfterWriteOriginal = lN - root.Doc.topLN
+			root.Quit(ctx)
+			return
+		}
+	}
+	root.Quit(ctx)
 }
