@@ -74,8 +74,11 @@ type Document struct {
 	// currentChunk represents the current chunk number.
 	currentChunk int
 
-	// headerLen is the actual header length when wrapped.
-	headerLen int
+	// headerHeight is the height of the header.
+	headerHeight int
+	// sectionHeaderHeight is the height of the section header.
+	sectionHeaderHeight int
+
 	// statusPos is the position of the status line.
 	statusPos int
 
@@ -104,8 +107,13 @@ type Document struct {
 	// columnCursor is the number of columns.
 	columnCursor int
 
-	// jumpTargetNum is the display position of search results.
-	jumpTargetNum int
+	// lastSearchLN is the last search line number.
+	lastSearchLN int
+	// showGotoF displays the specified line if it is true.
+	showGotoF bool
+
+	// jumpTargetHeight is the display position of search results.
+	jumpTargetHeight int
 	// jumpTargetSection is the display position of search results.
 	jumpTargetSection bool
 
@@ -130,8 +138,6 @@ type Document struct {
 	seekable bool
 	// Is it possible to reopen.
 	reopenable bool
-	// dupSectionHeader is true for lines that are duplicated in section header.
-	dupSectionHeader bool
 	// If nonMatch is true, non-matching lines are searched.
 	nonMatch bool
 }
@@ -178,9 +184,18 @@ type chunk struct {
 // LineC is one line of information.
 // Contains content, string, location information.
 type LineC struct {
-	lc  contents
+	// line contents.
+	lc contents
+	// string representation of the line.
 	str string
+	// for converting the width of str and lc.
 	pos widthPos
+	// valid is true if the line is valid.
+	valid bool
+	// The number of the section in the screen.
+	section int
+	// Line number within a section.
+	sectionNm int
 }
 
 // NewDocument returns Document.
@@ -193,12 +208,12 @@ func NewDocument() (*Document, error) {
 			TabWidth:        8,
 			MarkStyleWidth:  1,
 		},
-		ctlCh:            make(chan controlSpecifier),
-		memoryLimit:      100,
-		seekable:         true,
-		reopenable:       true,
-		store:            NewStore(),
-		dupSectionHeader: true,
+		ctlCh:        make(chan controlSpecifier),
+		memoryLimit:  100,
+		seekable:     true,
+		reopenable:   true,
+		store:        NewStore(),
+		lastSearchLN: -1,
 	}
 	if err := m.NewCache(); err != nil {
 		return nil, err
@@ -264,7 +279,7 @@ func STDINDocument() (*Document, error) {
 
 	m.seekable = false
 	m.reopenable = false
-	m.Caption = "(STDIN)"
+	m.FileName = "(STDIN)"
 	f, err := open("")
 	if err != nil {
 		return nil, err
@@ -405,12 +420,13 @@ func (m *Document) contents(lN int, tabWidth int) (contents, error) {
 
 // getLineC returns contents from line number and tabWidth.
 // If the line number does not exist, EOF content is returned.
-func (m *Document) getLineC(lN int, tabWidth int) (LineC, bool) {
+func (m *Document) getLineC(lN int, tabWidth int) LineC {
 	if line, ok := m.cache.Get(lN); ok {
 		lc := make(contents, len(line.lc))
 		copy(lc, line.lc)
 		line.lc = lc
-		return line, true
+		line.valid = true
+		return line
 	}
 
 	org, err := m.contents(lN, tabWidth)
@@ -418,10 +434,11 @@ func (m *Document) getLineC(lN int, tabWidth int) (LineC, bool) {
 		lc := make(contents, 1)
 		lc[0] = EOFContent
 		return LineC{
-			lc:  lc,
-			str: string(EOFC),
-			pos: widthPos{0: 0, 1: 1},
-		}, false
+			lc:    lc,
+			str:   string(EOFC),
+			pos:   widthPos{0: 0, 1: 1},
+			valid: false,
+		}
 	}
 	str, pos := ContentsToStr(org)
 	line := LineC{
@@ -436,7 +453,8 @@ func (m *Document) getLineC(lN int, tabWidth int) (LineC, bool) {
 	lc := make(contents, len(org))
 	copy(lc, org)
 	line.lc = lc
-	return line, true
+	line.valid = true
+	return line
 }
 
 // firstLine is the first line that excludes the SkipLines and Header.
@@ -450,6 +468,7 @@ func (m *Document) watchMode() {
 	if m.SectionDelimiter == "" {
 		m.setSectionDelimiter("^" + FormFeed)
 	}
+	m.SectionHeader = false
 	m.SectionStartPosition = 1
 	m.FollowSection = true
 }
