@@ -22,85 +22,94 @@ func (root *Root) eventLoop(ctx context.Context, quitChan chan<- struct{}) {
 	defer root.debugNumOfChunk()
 
 	for {
-		root.everyUpdate()
+		root.everyUpdate(ctx)
 		ev := root.Screen.PollEvent()
-		switch ev := ev.(type) {
-		case *eventAppQuit:
-			if root.Doc.documentType != DocHelp && root.Doc.documentType != DocLog {
-				close(quitChan)
-				return
-			}
-			// Help and logDoc return to Doc display.
-			root.toNormal()
-		case *eventReload:
-			root.reload(ev.m)
-		case *eventAppSuspend:
-			root.suspend()
-		case *eventUpdateEndNum:
-			root.updateEndNum()
-		case *eventDocument:
-			root.switchDocument(ev.docNum)
-		case *eventAddDocument:
-			root.addDocument(ev.m)
-		case *eventCloseDocument:
-			root.closeDocument()
-		case *eventCloseAllFilter:
-			root.closeAllFilter()
-		case *eventCopySelect:
-			root.copyToClipboard(ctx)
-		case *eventPaste:
-			root.pasteFromClipboard(ctx)
-		case *eventViewMode:
-			root.setViewMode(ev.value)
-		case *eventInputSearch:
-			root.firstSearch(ctx, ev.searchType)
-		case *eventNextSearch:
-			root.nextSearch(ctx, ev.str)
-		case *eventNextBackSearch:
-			root.nextBackSearch(ctx, ev.str)
-		case *eventSearchMove:
-			root.searchGo(ev.value)
-		case *eventGoto:
-			root.goLine(ev.value)
-		case *eventHeader:
-			root.setHeader(ev.value)
-		case *eventSkipLines:
-			root.setSkipLines(ev.value)
-		case *eventDelimiter:
-			root.setDelimiter(ev.value)
-		case *eventTabWidth:
-			root.setTabWidth(ev.value)
-		case *eventWatchInterval:
-			root.setWatchInterval(ev.value)
-		case *eventWriteBA:
-			root.setWriteBA(ev.value)
-		case *eventSectionDelimiter:
-			root.setSectionDelimiter(ev.value)
-		case *eventSectionStart:
-			root.setSectionStart(ev.value)
-		case *eventMultiColor:
-			root.setMultiColor(ev.value)
-		case *eventJumpTarget:
-			root.setJumpTarget(ev.value)
-		case *eventSaveBuffer:
-			root.saveBuffer(ev.value)
-		case *eventSectionNum:
-			root.setSectionNum(ev.value)
-
-		// tcell events
-		case *tcell.EventResize:
-			root.resize()
-		case *tcell.EventMouse:
-			root.mouseEvent(ev)
-		case *tcell.EventKey:
-			root.keyEvent(ctx, ev)
-		case nil:
+		if quit := root.event(ctx, ev); quit {
 			close(quitChan)
 			return
 		}
 	}
 }
 
+// event represents the event processing.
+func (root *Root) event(ctx context.Context, ev tcell.Event) bool {
+	switch ev := ev.(type) {
+	case *eventAppQuit:
+		if root.Doc.documentType != DocHelp && root.Doc.documentType != DocLog {
+			return true
+		}
+		// Help and logDoc return to Doc display.
+		root.toNormal(ctx)
+	case *eventReload:
+		root.reload(ev.m)
+	case *eventAppSuspend:
+		root.suspend(ctx)
+	case *eventUpdateEndNum:
+		root.updateEndNum()
+	case *eventDocument:
+		root.switchDocument(ctx, ev.docNum)
+	case *eventAddDocument:
+		root.addDocument(ctx, ev.m)
+	case *eventCloseDocument:
+		root.closeDocument(ctx)
+	case *eventCloseAllFilter:
+		root.closeAllFilter(ctx)
+	case *eventCopySelect:
+		root.copyToClipboard(ctx)
+	case *eventPaste:
+		root.pasteFromClipboard(ctx)
+	case *eventViewMode:
+		root.setViewMode(ctx, ev.value)
+	case *eventInputSearch:
+		root.firstSearch(ctx, ev.searchType)
+	case *eventSearch:
+		root.forwardSearch(ctx, ev.str, 0)
+	case *eventNextSearch:
+		root.forwardSearch(ctx, ev.str, 1)
+	case *eventNextBackSearch:
+		root.backSearch(ctx, ev.str, -1)
+	case *eventSearchMove:
+		root.searchGo(ctx, ev.value)
+	case *eventGoto:
+		root.goLine(ev.value)
+	case *eventHeader:
+		root.setHeader(ev.value)
+	case *eventSkipLines:
+		root.setSkipLines(ev.value)
+	case *eventDelimiter:
+		root.setDelimiter(ev.value)
+	case *eventTabWidth:
+		root.setTabWidth(ev.value)
+	case *eventWatchInterval:
+		root.setWatchInterval(ev.value)
+	case *eventWriteBA:
+		root.setWriteBA(ctx, ev.value)
+	case *eventSectionDelimiter:
+		root.setSectionDelimiter(ev.value)
+	case *eventSectionStart:
+		root.setSectionStart(ev.value)
+	case *eventMultiColor:
+		root.setMultiColor(ev.value)
+	case *eventJumpTarget:
+		root.setJumpTarget(ev.value)
+	case *eventSaveBuffer:
+		root.saveBuffer(ev.value)
+	case *eventSectionNum:
+		root.setSectionNum(ev.value)
+
+	// tcell events
+	case *tcell.EventResize:
+		root.resize(ctx)
+	case *tcell.EventMouse:
+		root.mouseEvent(ctx, ev)
+	case *tcell.EventKey:
+		root.keyEvent(ctx, ev)
+	case nil:
+		return true
+	}
+	return false
+}
+
 // sendGoto fires an eventGoto event that moves to the specified line.
 func (root *Root) sendGoto(num int) {
 	ev := &eventGoto{}
@@ -125,7 +134,7 @@ func (root *Root) MoveBottom() {
 }
 
 // everyUpdate is called every time before running the event.
-func (root *Root) everyUpdate() {
+func (root *Root) everyUpdate(ctx context.Context) {
 	// If tmpLN is set, set top position to position from bottom.
 	// This process is executed when temporary read is switched to normal read.
 	if n := atomic.SwapInt32(&root.Doc.tmpLN, 0); n > 0 {
@@ -134,14 +143,14 @@ func (root *Root) everyUpdate() {
 	}
 
 	root.Doc.width = root.scr.vWidth - root.scr.startX
-	root.Doc.height = root.Doc.statusPos - root.Doc.headerLen
+	root.Doc.height = root.Doc.statusPos - root.Doc.headerHeight
 
 	if root.General.FollowAll || root.Doc.FollowMode || root.Doc.FollowSection {
-		root.follow()
+		root.follow(ctx)
 	}
 
-	if !root.skipDraw {
-		root.draw()
+	if !root.skipDraw && root.Doc.height > 0 {
+		root.draw(ctx)
 	}
 	root.skipDraw = false
 
@@ -167,7 +176,7 @@ type eventAppQuit struct {
 }
 
 // Quit fires the eventAppQuit event.
-func (root *Root) Quit() {
+func (root *Root) Quit(context.Context) {
 	root.sendQuit()
 }
 
@@ -304,7 +313,7 @@ type eventReload struct {
 }
 
 // Reload fires the eventReload event.
-func (root *Root) Reload() {
+func (root *Root) Reload(context.Context) {
 	root.sendReload(root.Doc)
 }
 
@@ -324,7 +333,10 @@ func (root *Root) releaseEventBuffer() {
 
 // postEvent is a wrapper for tcell.Event.
 func (root *Root) postEvent(ev tcell.Event) {
-	if !root.checkScreen() {
+	if root == nil {
+		return
+	}
+	if root.Screen == nil {
 		return
 	}
 
@@ -333,13 +345,3 @@ func (root *Root) postEvent(ev tcell.Event) {
 		root.releaseEventBuffer()
 	}
 }
-
-// checkScreen returns true if the screen is ready.
-// checkScreen is used in case it is called directly from the outside.
-// True if called from the event loop.
-func (root *Root) checkScreen() bool {
-	if root == nil {
-		return false
-	}
-	return root.Screen != nil
-}
