@@ -1,6 +1,7 @@
 package oviewer
 
 import (
+	"context"
 	"fmt"
 	"io"
 	"strings"
@@ -80,6 +81,7 @@ const (
 	actionMultiColor     = "multi_color"
 	actionJumpTarget     = "jump_target"
 	actionSaveBuffer     = "save_buffer"
+	actionHideOther      = "hide_other"
 
 	inputCaseSensitive      = "input_casesensitive"
 	inputSmartCaseSensitive = "input_smart_casesensitive"
@@ -93,8 +95,8 @@ const (
 )
 
 // handlers returns a map of the action's handlers.
-func (root *Root) handlers() map[string]func() {
-	return map[string]func(){
+func (root *Root) handlers() map[string]func(context.Context) {
+	return map[string]func(context.Context){
 		actionExit:           root.Quit,
 		actionWriteBA:        root.setWriteBAMode,
 		actionCancel:         root.Cancel,
@@ -144,7 +146,7 @@ func (root *Root) handlers() map[string]func() {
 		actionMark:           root.addMark,
 		actionRemoveMark:     root.removeMark,
 		actionRemoveAllMark:  root.removeAllMark,
-		actionSearch:         root.setSearchMode,
+		actionSearch:         root.setForwardSearchMode,
 		actionBackSearch:     root.setBackSearchMode,
 		actionFilter:         root.setSearchFilterMode,
 		actionDelimiter:      root.setDelimiterMode,
@@ -163,6 +165,7 @@ func (root *Root) handlers() map[string]func() {
 		actionMultiColor:     root.setMultiColorMode,
 		actionJumpTarget:     root.setJumpTargetMode,
 		actionSaveBuffer:     root.setSaveBuffer,
+		actionHideOther:      root.toggleHideOtherSection,
 
 		inputCaseSensitive:      root.inputCaseSensitive,
 		inputSmartCaseSensitive: root.inputSmartCaseSensitive,
@@ -250,6 +253,7 @@ func defaultKeyBinds() KeyBind {
 		actionMultiColor:     {"."},
 		actionJumpTarget:     {"j"},
 		actionSaveBuffer:     {"S"},
+		actionHideOther:      {"alt+-"},
 
 		inputCaseSensitive:      {"alt+c"},
 		inputSmartCaseSensitive: {"alt+s"},
@@ -266,7 +270,7 @@ func defaultKeyBinds() KeyBind {
 // String returns keybind as a string for help.
 func (k KeyBind) String() string {
 	var b strings.Builder
-	writeHeader(&b, "Key binding")
+	writeHeaderTh(&b)
 	k.writeKeyBind(&b, actionExit, "quit")
 	k.writeKeyBind(&b, actionCancel, "cancel")
 	k.writeKeyBind(&b, actionWriteExit, "output screen and quit")
@@ -344,7 +348,8 @@ func (k KeyBind) String() string {
 	k.writeKeyBind(&b, actionPrevSection, "previous section")
 	k.writeKeyBind(&b, actionLastSection, "last section")
 	k.writeKeyBind(&b, actionFollowSection, "follow section mode toggle")
-	k.writeKeyBind(&b, actionSectionNum, "section header number")
+	k.writeKeyBind(&b, actionSectionNum, "number of section header lines")
+	k.writeKeyBind(&b, actionHideOther, "toggle hide other section")
 
 	writeHeader(&b, "Close and reload")
 	k.writeKeyBind(&b, actionCloseFile, "close file")
@@ -365,12 +370,16 @@ func (k KeyBind) String() string {
 	return b.String()
 }
 
+func writeHeaderTh(w io.Writer) {
+	fmt.Fprintf(w, " %-30s %s\n", "Key", "Action")
+}
+
 func writeHeader(w io.Writer, header string) {
 	fmt.Fprintf(w, "\n\t%s\n", header)
 }
 
 func (k KeyBind) writeKeyBind(w io.Writer, action string, detail string) {
-	fmt.Fprintf(w, " %-28s * %s\n", "["+strings.Join(k[action], "], [")+"]", detail)
+	fmt.Fprintf(w, " %-30s * %s\n", "["+strings.Join(k[action], "], [")+"]", detail)
 }
 
 // GetKeyBinds returns the current key mapping.
@@ -389,7 +398,7 @@ func GetKeyBinds(config Config) KeyBind {
 }
 
 // setHandlers sets keys to action handlers.
-func (root *Root) setHandlers(keyBind KeyBind) error {
+func (root *Root) setHandlers(ctx context.Context, keyBind KeyBind) error {
 	c := root.keyConfig
 	in := root.inputKeyConfig
 
@@ -402,12 +411,12 @@ func (root *Root) setHandlers(keyBind KeyBind) error {
 		}
 
 		if strings.HasPrefix(name, "input_") {
-			if err := setHandler(in, name, keys, handler); err != nil {
+			if err := setHandler(ctx, in, name, keys, handler); err != nil {
 				return err
 			}
 			continue
 		}
-		if err := setHandler(c, name, keys, handler); err != nil {
+		if err := setHandler(ctx, c, name, keys, handler); err != nil {
 			return err
 		}
 	}
@@ -415,37 +424,37 @@ func (root *Root) setHandlers(keyBind KeyBind) error {
 }
 
 // setHandler sets multiple keys in one action handler.
-func setHandler(c *cbind.Configuration, name string, keys []string, handler func()) error {
+func setHandler(ctx context.Context, c *cbind.Configuration, name string, keys []string, handler func(context.Context)) error {
 	for _, k := range keys {
 		mod, key, ch, err := cbind.Decode(k)
 		if err != nil {
-			return fmt.Errorf("%w [%s] for %s: %s", ErrFailedKeyBind, k, name, err)
+			return fmt.Errorf("%w [%s] for %s: %w", ErrFailedKeyBind, k, name, err)
 		}
 		if key == tcell.KeyRune {
-			c.SetRune(mod, ch, wrapEventHandler(handler))
+			c.SetRune(mod, ch, wrapEventHandler(ctx, handler))
 			// Added "shift+N" instead of 'N' to get it on windows.
 			if 0x21 <= ch && ch <= 0x60 {
-				c.SetRune(mod|tcell.ModShift, ch, wrapEventHandler(handler))
+				c.SetRune(mod|tcell.ModShift, ch, wrapEventHandler(ctx, handler))
 			}
 		} else {
 			// ctrl+h, Backspace and ctrl+Backspace can only be assigned one handler.
 			if key == tcell.KeyBackspace || key == tcell.KeyBackspace2 {
-				c.SetKey(tcell.ModNone, tcell.KeyBackspace, wrapEventHandler(handler))
-				c.SetKey(tcell.ModNone, tcell.KeyBackspace2, wrapEventHandler(handler))
-				c.SetKey(tcell.ModCtrl, tcell.KeyBackspace, wrapEventHandler(handler))
-				c.SetKey(tcell.ModCtrl, tcell.KeyBackspace2, wrapEventHandler(handler))
+				c.SetKey(tcell.ModNone, tcell.KeyBackspace, wrapEventHandler(ctx, handler))
+				c.SetKey(tcell.ModNone, tcell.KeyBackspace2, wrapEventHandler(ctx, handler))
+				c.SetKey(tcell.ModCtrl, tcell.KeyBackspace, wrapEventHandler(ctx, handler))
+				c.SetKey(tcell.ModCtrl, tcell.KeyBackspace2, wrapEventHandler(ctx, handler))
 				continue
 			}
-			c.SetKey(mod, key, wrapEventHandler(handler))
+			c.SetKey(mod, key, wrapEventHandler(ctx, handler))
 		}
 	}
 	return nil
 }
 
 // wrapEventHandler is a wrapper for matching func types.
-func wrapEventHandler(f func()) func(_ *tcell.EventKey) *tcell.EventKey {
+func wrapEventHandler(ctx context.Context, f func(context.Context)) func(_ *tcell.EventKey) *tcell.EventKey {
 	return func(_ *tcell.EventKey) *tcell.EventKey {
-		f()
+		f(ctx)
 		return nil
 	}
 }
