@@ -15,7 +15,7 @@ import (
 var WheelScrollNum = 2
 
 // mouseEvent handles mouse events.
-func (root *Root) mouseEvent(ev *tcell.EventMouse) {
+func (root *Root) mouseEvent(ctx context.Context, ev *tcell.EventMouse) {
 	button := ev.Buttons()
 	mod := ev.Modifiers()
 
@@ -26,33 +26,33 @@ func (root *Root) mouseEvent(ev *tcell.EventMouse) {
 
 	if button&tcell.WheelUp != 0 {
 		if horizontal {
-			root.wheelLeft()
+			root.wheelLeft(ctx)
 			return
 		}
-		root.wheelUp()
+		root.wheelUp(ctx)
 		return
 	}
 
 	if button&tcell.WheelDown != 0 {
 		if horizontal {
-			root.wheelRight()
+			root.wheelRight(ctx)
 			return
 		}
-		root.wheelDown()
+		root.wheelDown(ctx)
 		return
 	}
 
 	if button&tcell.WheelLeft != 0 {
-		root.wheelLeft()
+		root.wheelLeft(ctx)
 		return
 	}
 	if button&tcell.WheelRight != 0 {
-		root.wheelRight()
+		root.wheelRight(ctx)
 		return
 	}
 
-	if button != tcell.ButtonNone || root.mouseSelect {
-		root.selectRange(ev)
+	if button != tcell.ButtonNone || root.scr.mouseSelect {
+		root.selectRange(ctx, ev)
 		return
 	}
 
@@ -61,30 +61,30 @@ func (root *Root) mouseEvent(ev *tcell.EventMouse) {
 }
 
 // wheelUp moves the mouse wheel up.
-func (root *Root) wheelUp() {
+func (root *Root) wheelUp(context.Context) {
 	root.setMessage("")
 	root.moveUp(WheelScrollNum)
 }
 
 // wheelDown moves the mouse wheel down.
-func (root *Root) wheelDown() {
+func (root *Root) wheelDown(context.Context) {
 	root.setMessage("")
 	root.moveDown(WheelScrollNum)
 }
 
-func (root *Root) wheelRight() {
+func (root *Root) wheelRight(ctx context.Context) {
 	root.setMessage("")
 	if root.Doc.ColumnMode {
-		root.moveRightOne()
+		root.moveRightOne(ctx)
 	} else {
 		root.moveRight(root.Doc.HScrollWidthNum)
 	}
 }
 
-func (root *Root) wheelLeft() {
+func (root *Root) wheelLeft(ctx context.Context) {
 	root.setMessage("")
 	if root.Doc.ColumnMode {
-		root.moveLeftOne()
+		root.moveLeftOne(ctx)
 	} else {
 		root.moveLeft(root.Doc.HScrollWidthNum)
 	}
@@ -92,36 +92,36 @@ func (root *Root) wheelLeft() {
 
 // selectRange saves the position by selecting the range with the mouse.
 // The selection range is represented by (x1, y1), (x2, y2).
-func (root *Root) selectRange(ev *tcell.EventMouse) {
+func (root *Root) selectRange(ctx context.Context, ev *tcell.EventMouse) {
 	button := ev.Buttons()
 	if button == tcell.ButtonMiddle {
-		root.Paste()
+		root.Paste(ctx)
 		return
 	}
 
-	if !root.mouseSelect && button == tcell.ButtonPrimary {
+	if !root.scr.mouseSelect && button == tcell.ButtonPrimary {
 		root.setMessage("")
 		if ev.Modifiers()&tcell.ModCtrl != 0 {
-			root.mouseRectangle = true
+			root.scr.mouseRectangle = true
 		} else {
-			root.mouseRectangle = false
+			root.scr.mouseRectangle = false
 		}
-		root.mouseSelect = true
-		root.mousePressed = true
-		root.x1, root.y1 = ev.Position()
+		root.scr.mouseSelect = true
+		root.scr.mousePressed = true
+		root.scr.x1, root.scr.y1 = ev.Position()
 	}
 
-	if root.mousePressed {
-		root.x2, root.y2 = ev.Position()
+	if root.scr.mousePressed {
+		root.scr.x2, root.scr.y2 = ev.Position()
 	}
 
-	if root.mouseSelect {
+	if root.scr.mouseSelect {
 		if button == tcell.ButtonNone {
-			root.mousePressed = false
-		} else if !root.mousePressed {
+			root.scr.mousePressed = false
+		} else if !root.scr.mousePressed {
 			root.resetSelect()
 			if button == tcell.ButtonPrimary || button == tcell.ButtonSecondary {
-				root.CopySelect()
+				root.CopySelect(ctx)
 			}
 		}
 	}
@@ -129,8 +129,8 @@ func (root *Root) selectRange(ev *tcell.EventMouse) {
 
 // resetSelect resets the selection.
 func (root *Root) resetSelect() {
-	root.mouseSelect = false
-	root.mousePressed = false
+	root.scr.mouseSelect = false
+	root.scr.mousePressed = false
 }
 
 // eventCopySelect represents a mouse select event.
@@ -139,7 +139,7 @@ type eventCopySelect struct {
 }
 
 // CopySelect fires the eventCopySelect event.
-func (root *Root) CopySelect() {
+func (root *Root) CopySelect(context.Context) {
 	root.sendCopySelect()
 }
 
@@ -151,10 +151,10 @@ func (root *Root) sendCopySelect() {
 
 // copyToClipboard writes the selection to the clipboard.
 func (root *Root) copyToClipboard(_ context.Context) {
-	x1 := root.x1
-	x2 := root.x2
-	y1 := root.y1
-	y2 := root.y2
+	x1 := root.scr.x1
+	x2 := root.scr.x2
+	y1 := root.scr.y1
+	y2 := root.scr.y2
 
 	if y2 < y1 {
 		y1, y2 = y2, y1
@@ -165,16 +165,16 @@ func (root *Root) copyToClipboard(_ context.Context) {
 			x1, x2 = x2, x1
 		}
 	}
-	buff, err := root.rangeToString(x1, y1, x2, y2)
+	str, err := root.rangeToString(x1, y1, x2, y2)
 	if err != nil {
 		root.debugMessage(fmt.Sprintf("copyToClipboard: %s", err.Error()))
 		return
 	}
 
-	if len(buff) == 0 {
+	if len(str) == 0 {
 		return
 	}
-	if err := clipboard.WriteAll(buff); err != nil {
+	if err := clipboard.WriteAll(str); err != nil {
 		log.Printf("copyToClipboard: %v", err)
 	}
 	root.setMessage("Copy")
@@ -185,7 +185,7 @@ type eventPaste struct {
 }
 
 // Paste fires the eventPaste event.
-func (root *Root) Paste() {
+func (root *Root) Paste(context.Context) {
 	root.sendPaste()
 }
 
@@ -196,10 +196,9 @@ func (root *Root) sendPaste() {
 }
 
 // pasteFromClipboard writes a string from the clipboard.
-func (root *Root) pasteFromClipboard(_ context.Context) {
+func (root *Root) pasteFromClipboard(context.Context) {
 	input := root.input
-	switch input.Event.Mode() {
-	case Normal:
+	if input.Event.Mode() == Normal {
 		return
 	}
 
@@ -217,52 +216,8 @@ func (root *Root) pasteFromClipboard(_ context.Context) {
 	input.cursorX += runewidth.StringWidth(str)
 }
 
-// drawSelect highlights the selection.
-// Multi-line selection is included until the end of the line,
-// but if the rectangle flag is true, the rectangle will be the range.
-func (root *Root) drawSelect(x1, y1, x2, y2 int, sel bool) {
-	if y2 < y1 {
-		y1, y2 = y2, y1
-		x1, x2 = x2, x1
-	}
-	if y1 == y2 {
-		if x2 < x1 {
-			x1, x2 = x2, x1
-		}
-		root.reverseLine(y1, x1, x2+1, sel)
-		return
-	}
-	if root.mouseRectangle {
-		for y := y1; y <= y2; y++ {
-			root.reverseLine(y, x1, x2+1, sel)
-		}
-		return
-	}
-
-	root.reverseLine(y1, x1, root.scr.vWidth, sel)
-	for y := y1 + 1; y < y2; y++ {
-		root.reverseLine(y, 0, root.scr.vWidth, sel)
-	}
-	root.reverseLine(y2, 0, x2+1, sel)
-}
-
-// reverseLine reverses one line.
-func (root *Root) reverseLine(y int, start int, end int, sel bool) {
-	if start >= end {
-		return
-	}
-	for x := start; x < end; x++ {
-		mainc, combc, style, width := root.Screen.GetContent(x, y)
-		style = style.Reverse(sel)
-		root.Screen.SetContent(x, y, mainc, combc, style)
-		if width == 2 {
-			x++
-		}
-	}
-}
-
 func (root *Root) rangeToString(x1, y1, x2, y2 int) (string, error) {
-	if root.mouseRectangle {
+	if root.scr.mouseRectangle {
 		return root.scr.rectangleToString(root.Doc, x1, y1, x2, y2)
 	}
 	return root.scr.lineRangeToString(root.Doc, x1, y1, x2, y2)
@@ -277,8 +232,8 @@ func (scr SCR) lineRangeToString(m *Document, x1, y1, x2, y2 int) (string, error
 	var buff strings.Builder
 
 	l1 := scr.lineNumber(y1)
-	line1, valid := m.getLineC(l1.number, m.TabWidth)
-	if !valid {
+	line1 := m.getLineC(l1.number, m.TabWidth)
+	if !line1.valid {
 		return "", ErrOutOfRange
 	}
 	wx1 := scr.branchWidth(line1.lc, l1.wrap)
@@ -292,8 +247,8 @@ func (scr SCR) lineRangeToString(m *Document, x1, y1, x2, y2 int) (string, error
 			break
 		}
 	}
-	line2, valid := m.getLineC(l2.number, m.TabWidth)
-	if !valid {
+	line2 := m.getLineC(l2.number, m.TabWidth)
+	if !line2.valid {
 		return "", ErrOutOfRange
 	}
 	wx2 := scr.branchWidth(line2.lc, l2.wrap)
@@ -325,8 +280,8 @@ func (scr SCR) lineRangeToString(m *Document, x1, y1, x2, y2 int) (string, error
 		if ln.number == l1.number || ln.number == l2.number || ln.wrap > 0 {
 			continue
 		}
-		line, valid := m.getLineC(ln.number, m.TabWidth)
-		if !valid {
+		line := m.getLineC(ln.number, m.TabWidth)
+		if !line.valid {
 			break
 		}
 		str := scr.selectLine(line, 0, -1)
@@ -355,8 +310,8 @@ func (scr SCR) rectangleToString(m *Document, x1, y1, x2, y2 int) (string, error
 	var buff strings.Builder
 	for y := y1; y <= y2; y++ {
 		ln := scr.lineNumber(y)
-		line, valid := m.getLineC(ln.number, m.TabWidth)
-		if !valid {
+		line := m.getLineC(ln.number, m.TabWidth)
+		if !line.valid {
 			return "", fmt.Errorf("rectangleToString: %w", ErrOutOfRange)
 		}
 		wx := scr.branchWidth(line.lc, ln.wrap)
