@@ -6,7 +6,6 @@ import (
 	"fmt"
 	"io"
 	"log"
-	"math"
 	"os"
 	"os/signal"
 	"path/filepath"
@@ -46,6 +45,9 @@ type Root struct {
 	// Original string.
 	OriginStr string
 
+	// searchOpt is the search option.
+	searchOpt string
+
 	// message is the message to display.
 	message string
 	// cancelKeys represents the cancellation key string.
@@ -53,7 +55,8 @@ type Root struct {
 
 	// DocList is the list of documents.
 	DocList []*Document
-	scr     SCR
+	// scr contains the screen information.
+	scr SCR
 	// Config is the configuration of ov.
 	Config
 	// Original position at the start of search.
@@ -63,29 +66,20 @@ type Root struct {
 	CurrentDoc int
 	// minStartX is the minimum start position of x.
 	minStartX int
-	// x1, y1, x2, y2 are the coordinates selected by the mouse.
-	x1 int
-	y1 int
-	x2 int
-	y2 int
 
 	// mu controls the RWMutex.
 	mu sync.RWMutex
 
 	// skipDraw is set to true when the mouse cursor just moves (no event occurs).
 	skipDraw bool
-	// mousePressed is a flag when the mouse selection button is pressed.
-	mousePressed bool
-	// mouseSelect is a flag with mouse selection.
-	mouseSelect bool
-	// mouseRectangle is a flag for rectangle selection.
-	mouseRectangle bool
 	// Show document number
 	showDocNum bool
 }
 
 // SCR contains the screen information.
 type SCR struct {
+	// lines is the lines of the screen.
+	lines map[int]LineC
 	// numbers is the line information of the currently displayed screen.
 	// numbers (number of logical numbers and number of wrapping numbers) from y on the screen.
 	numbers []LineNumber
@@ -95,8 +89,27 @@ type SCR struct {
 	vHeight int
 	// startX is the start position of x.
 	startX int
-	// Process as a section header if the remaining value is 1 or more.
-	sectionHeaderLeft int
+
+	// HeaderLN is the number of header lines.
+	headerLN int
+	// headerEnd is the end of the header.
+	headerEnd int
+	// sectionHeaderLN is the number of section headers.
+	sectionHeaderLN int
+	// sectionHeaderEnd is the end of the section header.
+	sectionHeaderEnd int
+
+	// x1, y1, x2, y2 are the coordinates selected by the mouse.
+	x1 int
+	y1 int
+	x2 int
+	y2 int
+	// mouseSelect is a flag with mouse selection.
+	mouseSelect bool
+	// mousePressed is a flag when the mouse selection button is pressed.
+	mousePressed bool
+	// mouseRectangle is a flag for rectangle selection.
+	mouseRectangle bool
 }
 
 // LineNumber is Number of logical lines and number of wrapping lines on the screen.
@@ -129,7 +142,7 @@ type general struct {
 
 	// TabWidth is tab stop num.
 	TabWidth int
-	// HeaderLen is number of header rows to be fixed.
+	// Header is number of header lines to be fixed.
 	Header int
 	// SkipLines is the rows to skip.
 	SkipLines int
@@ -169,6 +182,8 @@ type general struct {
 	PlainMode bool
 	// SectionHeader is whether to display the section header.
 	SectionHeader bool
+	// HideOtherSection is whether to hide other sections.
+	HideOtherSection bool
 }
 
 // OVPromptConfigNormal is the normal prompt setting.
@@ -361,6 +376,8 @@ var (
 	ErrNoColumn = errors.New("no column")
 	// ErrNoDelimiter indicates that the line containing the delimiter could not be found.
 	ErrNoDelimiter = errors.New("no delimiter")
+	// ErrNoMoreSection indicates that the section could not be found.
+	ErrNoMoreSection = errors.New("no more section")
 	// ErrOverScreen indicates that the specified screen is out of range.
 	ErrOverScreen = errors.New("over screen")
 	// ErrOutOfChunk indicates that the specified Chunk is out of range.
@@ -568,9 +585,9 @@ func (root *Root) sendRequest(m *Document, request request) {
 }
 
 // setKeyConfig sets key bindings.
-func (root *Root) setKeyConfig() (map[string][]string, error) {
+func (root *Root) setKeyConfig(ctx context.Context) (map[string][]string, error) {
 	keyBind := GetKeyBinds(root.Config)
-	if err := root.setHandlers(keyBind); err != nil {
+	if err := root.setHandlers(ctx, keyBind); err != nil {
 		return nil, err
 	}
 
@@ -584,12 +601,16 @@ func (root *Root) setKeyConfig() (map[string][]string, error) {
 }
 
 // SetKeyHandler assigns a new key handler.
-func (root *Root) SetKeyHandler(name string, keys []string, handler func()) error {
-	return setHandler(root.keyConfig, name, keys, handler)
+func (root *Root) SetKeyHandler(ctx context.Context, name string, keys []string, handler func(context.Context)) error {
+	return setHandler(ctx, root.keyConfig, name, keys, handler)
 }
 
 // Run starts the terminal pager.
 func (root *Root) Run() error {
+	ctx := context.Background()
+	ctx, cancel := context.WithCancel(ctx)
+	defer cancel()
+
 	defer root.Close()
 	watcher, err := fsnotify.NewWatcher()
 	if err != nil {
@@ -600,7 +621,7 @@ func (root *Root) Run() error {
 
 	// Do not set the key bindings in NewOviewer
 	// because it is done after loading the config.
-	keyBind, err := root.setKeyConfig()
+	keyBind, err := root.setKeyConfig(ctx)
 	if err != nil {
 		return err
 	}
@@ -638,7 +659,7 @@ func (root *Root) Run() error {
 		log.Printf("open [%d]%s%s", n, doc.FileName, w)
 	}
 
-	root.ViewSync()
+	root.ViewSync(ctx)
 	// Exit if fits on screen
 	if root.QuitSmall && root.docSmall() {
 		root.IsWriteOriginal = true
@@ -652,9 +673,6 @@ func (root *Root) Run() error {
 
 	quitChan := make(chan struct{})
 
-	ctx := context.Background()
-	ctx, cancel := context.WithCancel(ctx)
-	defer cancel()
 	go func() {
 		// Undo screen when goroutine panic.
 		defer func() {
@@ -817,12 +835,12 @@ func applyStyle(style tcell.Style, s OVStyle) tcell.Style {
 	}
 	if s.OverLine {
 		// tcell does not support overline.
-		//style = style.OverLine(true)
+		// style = style.OverLine(true)
 		style = style.Underline(true)
 	}
 	if s.UnOverLine {
 		// tcell does not support unOverline.
-		//style = style.UnOverLine(false)
+		// style = style.UnOverLine(false)
 		style = style.Underline(false)
 	}
 	return style
@@ -899,24 +917,10 @@ func mergeGeneral(src general, dst general) general {
 	if dst.Caption != "" {
 		src.Caption = dst.Caption
 	}
-	return src
-}
-
-// prepareView prepares when the screen size is changed.
-func (root *Root) prepareView() {
-	screen := root.Screen
-	root.scr.vWidth, root.scr.vHeight = screen.Size()
-
-	// Do not allow size 0.
-	root.scr.vWidth = max(root.scr.vWidth, 1)
-	root.scr.vHeight = max(root.scr.vHeight, 1)
-	root.Doc.statusPos = root.scr.vHeight - statusLine
-
-	num := int(math.Round(calculatePosition(root.scr.vWidth, root.Doc.HScrollWidth)))
-	root.Doc.HScrollWidthNum = max(num, 1)
-	if len(root.scr.numbers) != root.scr.vHeight+1 {
-		root.scr.numbers = make([]LineNumber, root.scr.vHeight+1)
+	if dst.HideOtherSection {
+		src.HideOtherSection = dst.HideOtherSection
 	}
+	return src
 }
 
 // docSmall returns with bool whether the file to display fits on the screen.
@@ -924,7 +928,7 @@ func (root *Root) docSmall() bool {
 	if len(root.DocList) > 1 {
 		return false
 	}
-	root.prepareView()
+	root.prepareScreen()
 	m := root.Doc
 	if !m.BufEOF() {
 		return false
@@ -946,18 +950,39 @@ func (root *Root) docSmall() bool {
 
 // WriteOriginal writes to the original terminal.
 func (root *Root) WriteOriginal() {
+	output := os.Stdout
+	root.writeOriginal(output)
+}
+
+func (root *Root) writeOriginal(output io.Writer) {
 	m := root.Doc
 	if m.bottomLN == 0 {
 		m.bottomLN = m.BufEndNum()
 	}
 
-	start := max(0, m.topLN-(m.SectionHeaderNum+root.BeforeWriteOriginal))
+	// header
+	header := max(0, root.scr.headerLN)
+	headerEnd := max(0, root.scr.headerEnd)
+	if root.Doc.headerHeight > 0 {
+		if err := m.Export(output, header, headerEnd-1); err != nil {
+			log.Println(err)
+		}
+	}
+	// section header
+	secAdd := 0
+	if m.sectionHeaderHeight > 0 {
+		if err := m.Export(output, root.scr.sectionHeaderLN, root.scr.sectionHeaderEnd-1); err != nil {
+			log.Println(err)
+		}
+		secAdd = m.SectionHeaderNum
+	}
+	// body
+	start := max(m.topLN+m.firstLine()+secAdd, root.scr.sectionHeaderEnd) - root.BeforeWriteOriginal
 	end := m.bottomLN - 1
 	if root.AfterWriteOriginal != 0 {
 		end = m.topLN + root.AfterWriteOriginal - 1
 	}
-
-	if err := m.Export(os.Stdout, start, end); err != nil {
+	if err := m.Export(output, start, end); err != nil {
 		log.Println(err)
 	}
 }
@@ -974,6 +999,9 @@ func (root *Root) WriteLog() {
 
 // lineNumber returns the line information from y on the screen.
 func (scr SCR) lineNumber(y int) LineNumber {
+	if len(scr.numbers) == 0 {
+		return LineNumber{}
+	}
 	if y >= 0 && y <= len(scr.numbers) {
 		return scr.numbers[y]
 	}
