@@ -23,11 +23,11 @@ import (
 // if the search word matches the argument string.
 type Searcher interface {
 	// Match searches for bytes.
-	Match([]byte) bool
+	Match(target []byte) bool
 	// MatchString searches for strings.
-	MatchString(string) bool
+	MatchString(target string) bool
 	// FindAll searches for strings and returns the index of the match.
-	FindAll(string) [][]int
+	FindAll(target string) [][]int
 	// String returns the search word.
 	String() string
 }
@@ -38,21 +38,21 @@ type searchWord struct {
 }
 
 // searchWord Match is a case-insensitive search for bytes.
-func (substr searchWord) Match(s []byte) bool {
-	s = stripEscapeSequenceBytes(s)
-	return bytes.Contains(bytes.ToLower(s), []byte(substr.word))
+func (substr searchWord) Match(target []byte) bool {
+	target = stripEscapeSequenceBytes(target)
+	return bytes.Contains(bytes.ToLower(target), []byte(substr.word))
 }
 
 // searchWord MatchString is a case-insensitive search for string.
-func (substr searchWord) MatchString(s string) bool {
-	s = stripEscapeSequenceString(s)
-	return strings.Contains(strings.ToLower(s), substr.word)
+func (substr searchWord) MatchString(target string) bool {
+	target = stripEscapeSequenceString(target)
+	return strings.Contains(strings.ToLower(target), substr.word)
 }
 
 // searchWord FindAll searches for strings and returns the index of the match.
-func (substr searchWord) FindAll(s string) [][]int {
-	s = strings.ToLower(s)
-	return allStringIndex(s, substr.word)
+func (substr searchWord) FindAll(target string) [][]int {
+	target = strings.ToLower(target)
+	return allStringIndex(target, substr.word)
 }
 
 // searchWord String returns the search word.
@@ -66,20 +66,20 @@ type sensitiveWord struct {
 }
 
 // sensitiveWord Match is a case-sensitive search for bytes.
-func (substr sensitiveWord) Match(s []byte) bool {
-	s = stripEscapeSequenceBytes(s)
-	return bytes.Contains(s, []byte(substr.word))
+func (substr sensitiveWord) Match(target []byte) bool {
+	target = stripEscapeSequenceBytes(target)
+	return bytes.Contains(target, []byte(substr.word))
 }
 
 // sensitiveWord MatchString is a case-sensitive search for string.
-func (substr sensitiveWord) MatchString(s string) bool {
-	s = stripEscapeSequenceString(s)
-	return strings.Contains(s, substr.word)
+func (substr sensitiveWord) MatchString(target string) bool {
+	target = stripEscapeSequenceString(target)
+	return strings.Contains(target, substr.word)
 }
 
 // sensitiveWord FindAll searches for strings and returns the index of the match.
-func (substr sensitiveWord) FindAll(s string) [][]int {
-	return allStringIndex(s, substr.word)
+func (substr sensitiveWord) FindAll(target string) [][]int {
+	return allStringIndex(target, substr.word)
 }
 
 // String returns the search word.
@@ -94,20 +94,20 @@ type regexpWord struct {
 }
 
 // regexpWord Match is a regular expression search for bytes.
-func (substr regexpWord) Match(s []byte) bool {
-	s = stripEscapeSequenceBytes(s)
-	return substr.regexp.Match(s)
+func (substr regexpWord) Match(target []byte) bool {
+	target = stripEscapeSequenceBytes(target)
+	return substr.regexp.Match(target)
 }
 
 // regexpWord MatchString is a regular expression search for string.
-func (substr regexpWord) MatchString(s string) bool {
-	s = stripEscapeSequenceString(s)
-	return substr.regexp.MatchString(s)
+func (substr regexpWord) MatchString(target string) bool {
+	target = stripEscapeSequenceString(target)
+	return substr.regexp.MatchString(target)
 }
 
 // regexpWord FindAll searches for strings and returns the index of the match.
-func (substr regexpWord) FindAll(s string) [][]int {
-	return substr.regexp.FindAllStringIndex(s, -1)
+func (substr regexpWord) FindAll(target string) [][]int {
+	return substr.regexp.FindAllStringIndex(target, -1)
 }
 
 // regexpWord String returns the search word.
@@ -119,21 +119,21 @@ func (substr regexpWord) String() string {
 var stripRegexpES = regexp.MustCompile("(\x1b\\[[\\d;*]*m)|.\b")
 
 // stripEscapeSequenceString strips if it contains escape sequences.
-func stripEscapeSequenceString(s string) string {
-	// Remove EscapeSequence.
-	if strings.ContainsAny(s, "\x1b\b") {
-		s = stripRegexpES.ReplaceAllString(s, "")
+func stripEscapeSequenceString(src string) string {
+	if !strings.ContainsAny(src, "\x1b\b") {
+		return src
 	}
-	return s
+	// Remove EscapeSequence.
+	return stripRegexpES.ReplaceAllString(src, "")
 }
 
 // stripEscapeSequence strips if it contains escape sequences.
-func stripEscapeSequenceBytes(s []byte) []byte {
-	// Remove EscapeSequence.
-	if bytes.ContainsAny(s, "\x1b\b") {
-		s = stripRegexpES.ReplaceAll(s, []byte(""))
+func stripEscapeSequenceBytes(src []byte) []byte {
+	if !bytes.ContainsAny(src, "\x1b\b") {
+		return src
 	}
-	return s
+	// Remove EscapeSequence.
+	return stripRegexpES.ReplaceAll(src, []byte(""))
 }
 
 // NewSearcher returns the Searcher interface suitable for the search term.
@@ -217,8 +217,8 @@ func (root *Root) searchPosition(str string) [][]int {
 
 // searchXPos returns the x position of the first match.
 func (root *Root) searchXPos(lineNum int) int {
-	line, _ := root.Doc.getLineC(lineNum, root.Doc.TabWidth)
-	indexes := root.searchPosition(line.str)
+	line := root.Doc.getLineC(lineNum, root.Doc.TabWidth)
+	indexes := root.searcher.FindAll(line.str)
 	if len(indexes) == 0 {
 		return 0
 	}
@@ -258,14 +258,14 @@ func (root *Root) setSearcher(word string, caseSensitive bool) Searcher {
 }
 
 // searchMove searches forward/backward and moves to the nearest matching line.
-func (root *Root) searchMove(ctx context.Context, forward bool, lineNum int, searcher Searcher) {
+func (root *Root) searchMove(ctx context.Context, forward bool, lineNum int, searcher Searcher) bool {
 	if searcher == nil {
 		if root.Doc.jumpTargetSection {
-			root.Doc.jumpTargetNum = 0
+			root.Doc.jumpTargetHeight = 0
 		}
-		return
+		return false
 	}
-	word := root.searcher.String()
+	word := searcher.String()
 	root.setMessagef("search:%v (%v)Cancel", word, strings.Join(root.cancelKeys, ","))
 	eg, ctx := errgroup.WithContext(ctx)
 	ctx, cancel := context.WithCancel(ctx)
@@ -287,12 +287,13 @@ func (root *Root) searchMove(ctx context.Context, forward bool, lineNum int, sea
 
 	if err := eg.Wait(); err != nil {
 		root.setMessageLog(err.Error())
-		return
+		return false
 	}
 	root.setMessagef("search:%v", word)
+	return true
 }
 
-// searchLine is a forward/backward search wrap function
+// searchLine is a forward/backward search wrap function.
 func (m *Document) searchLine(ctx context.Context, searcher Searcher, forward bool, lineNum int) (int, error) {
 	if forward {
 		return m.SearchLine(ctx, searcher, lineNum)
@@ -474,17 +475,9 @@ func (root *Root) cancelWait(cancel context.CancelFunc) error {
 	}
 
 	c := cbind.NewConfiguration()
-
-	for _, k := range root.cancelKeys {
-		mod, key, ch, err := cbind.Decode(k)
-		if err != nil {
-			return fmt.Errorf("%w [%s] for cancel: %s", ErrFailedKeyBind, k, err)
-		}
-		if key == tcell.KeyRune {
-			c.SetRune(mod, ch, cancelApp)
-		} else {
-			c.SetKey(mod, key, cancelApp)
-		}
+	c, err := cancelKeys(c, root.cancelKeys, cancelApp)
+	if err != nil {
+		return err
 	}
 	// Allow only some events while searching.
 	for {
@@ -497,12 +490,27 @@ func (root *Root) cancelWait(cancel context.CancelFunc) error {
 		case *eventUpdateEndNum:
 			root.updateEndNum()
 		default:
-			root.setMessageLogf("unexpected event %#v", ev)
-			return nil
+			// ignore other events.
 		}
 	}
 }
 
+// cancelKeys sets the cancel key.
+func cancelKeys(c *cbind.Configuration, cancelKeys []string, cancelApp func(_ *tcell.EventKey) *tcell.EventKey) (*cbind.Configuration, error) {
+	for _, k := range cancelKeys {
+		mod, key, ch, err := cbind.Decode(k)
+		if err != nil {
+			return c, fmt.Errorf("%w [%s] for cancel: %w", ErrFailedKeyBind, k, err)
+		}
+		if key == tcell.KeyRune {
+			c.SetRune(mod, ch, cancelApp)
+		} else {
+			c.SetKey(mod, key, cancelApp)
+		}
+	}
+	return c, nil
+}
+
 // eventSearchQuit represents a search quit event.
 type eventSearchQuit struct {
 	tcell.EventTime
@@ -588,52 +596,49 @@ func (root *Root) returnStartPosition() int {
 
 // startSearchLN returns the start position of the search.
 func (root *Root) startSearchLN() int {
-	l := root.scr.lineNumber(root.Doc.headerLen + root.Doc.jumpTargetNum)
-	if l.number-root.Doc.topLN > root.Doc.topLN {
-		return 0
+	top := root.Doc.topLN + root.Doc.firstLine()
+	current := root.scr.lineNumber(root.Doc.headerHeight + root.Doc.sectionHeaderHeight + root.Doc.jumpTargetHeight)
+	if root.Doc.topLN == 0 && root.Doc.jumpTargetHeight > 0 {
+		top = top - current.number
 	}
-	return l.number
+
+	if root.Doc.lastSearchLN < 0 {
+		return top
+	}
+	if top > root.Doc.lastSearchLN || root.Doc.bottomLN < root.Doc.lastSearchLN {
+		return top
+	}
+	if root.Doc.lastSearchLN < current.number-root.Doc.SectionHeaderNum {
+		return top
+	}
+	if root.Doc.lastSearchLN > current.number+root.Doc.SectionHeaderNum {
+		return top
+	}
+	return root.Doc.lastSearchLN
 }
 
 // firstSearch performs the first search immediately after the input.
 func (root *Root) firstSearch(ctx context.Context, t searchType) {
 	switch t {
 	case forward:
-		root.firstForwardSearch(ctx)
-		return
+		root.forwardSearch(ctx, root.input.value, 0)
 	case backward:
-		root.firstBackSearch(ctx)
-		return
+		root.backSearch(ctx, root.input.value, 0)
 	case filter:
 		root.filter(ctx)
 	}
 }
 
-// firstForwardSearch performs the first forward search immediately after the input.
-func (root *Root) firstForwardSearch(ctx context.Context) {
-	searcher := root.setSearcher(root.input.value, root.Config.CaseSensitive)
-	root.searchMove(ctx, true, root.startSearchLN(), searcher)
-}
-
-// nextSearch performs the next search.
-func (root *Root) nextSearch(ctx context.Context, str string) {
+// forwardSearch performs the forward search.
+func (root *Root) forwardSearch(ctx context.Context, str string, next int) {
 	searcher := root.setSearcher(str, root.Config.CaseSensitive)
-	l := root.scr.lineNumber(root.Doc.headerLen + root.Doc.jumpTargetNum)
-	root.searchMove(ctx, true, l.number+1, searcher)
+	root.searchMove(ctx, true, root.startSearchLN()+next, searcher)
 }
 
-// firstBackSearch performs the first back search immediately after the input.
-func (root *Root) firstBackSearch(ctx context.Context) {
-	searcher := root.setSearcher(root.input.value, root.Config.CaseSensitive)
-	l := root.scr.lineNumber(root.Doc.headerLen)
-	root.searchMove(ctx, false, l.number, searcher)
-}
-
-// nextBackSearch performs the next back search.
-func (root *Root) nextBackSearch(ctx context.Context, str string) {
+// backSearch performs the back search.
+func (root *Root) backSearch(ctx context.Context, str string, next int) {
 	searcher := root.setSearcher(str, root.Config.CaseSensitive)
-	l := root.scr.lineNumber(root.Doc.headerLen + root.Doc.jumpTargetNum)
-	root.searchMove(ctx, false, l.number-1, searcher)
+	root.searchMove(ctx, false, root.startSearchLN()+next, searcher)
 }
 
 // eventNextSearch represents search event.
@@ -643,7 +648,7 @@ type eventNextSearch struct {
 }
 
 // sendNextSearch fires the eventNextSearch event.
-func (root *Root) sendNextSearch() {
+func (root *Root) sendNextSearch(context.Context) {
 	if root.searcher == nil {
 		return
 	}
@@ -661,7 +666,7 @@ type eventNextBackSearch struct {
 }
 
 // sendNextBackSearch fires the eventNextBackSearch event.
-func (root *Root) sendNextBackSearch() {
+func (root *Root) sendNextBackSearch(context.Context) {
 	if root.searcher == nil {
 		return
 	}
@@ -676,11 +681,18 @@ func (root *Root) sendNextBackSearch() {
 // This is for calling Search from the outside.
 // Normally, the event is executed from Confirm.
 func (root *Root) Search(str string) {
-	root.sendForwardSearch(str)
+	root.sendSearch(str)
 }
 
-func (root *Root) sendForwardSearch(str string) {
-	ev := &eventNextSearch{}
+// eventSearch represents search event.
+type eventSearch struct {
+	tcell.EventTime
+	str string
+}
+
+// sendSearch fires the eventSearch event.
+func (root *Root) sendSearch(str string) {
+	ev := &eventSearch{}
 	ev.str = str
 	ev.SetEventNow()
 	root.postEvent(ev)
