@@ -1,12 +1,14 @@
 package oviewer
 
 import (
+	"context"
 	"path/filepath"
 	"reflect"
 	"regexp"
 	"strings"
 	"testing"
 
+	"code.rocketnine.space/tslocum/cbind"
 	"github.com/gdamore/tcell/v2"
 )
 
@@ -730,12 +732,7 @@ func TestDocument_searchChunk(t *testing.T) {
 		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			t.Parallel()
-			m, err := OpenDocument(tt.fileName)
-			if err != nil {
-				t.Fatal(err)
-			}
-			for !m.BufEOF() {
-			}
+			m := docFileReadHelper(t, tt.fileName)
 			got, err := m.searchChunk(tt.args.chunkNum, tt.args.searcher)
 			if (err != nil) != tt.wantErr {
 				t.Errorf("Document.searchChunk() error = %v, wantErr %v", err, tt.wantErr)
@@ -866,3 +863,279 @@ func Test_search_String(t *testing.T) {
 		})
 	}
 }
+
+func TestRoot_startSearchLN(t *testing.T) {
+	tcellNewScreen = fakeScreen
+	defer func() {
+		tcellNewScreen = tcell.NewScreen
+	}()
+	type fields struct {
+		fileName         string
+		topLN            int
+		lastSearchLN     int
+		sectionHeaderNum int
+	}
+	tests := []struct {
+		name   string
+		fields fields
+		want   int
+	}{
+		{
+			name: "test1",
+			fields: fields{
+				fileName:         filepath.Join(testdata, "test3.txt"),
+				topLN:            10,
+				lastSearchLN:     -1,
+				sectionHeaderNum: 0,
+			},
+			want: 10,
+		},
+		{
+			name: "test lastSearchLN",
+			fields: fields{
+				fileName:         filepath.Join(testdata, "test3.txt"),
+				topLN:            10,
+				lastSearchLN:     11,
+				sectionHeaderNum: 3,
+			},
+			want: 11,
+		},
+		{
+			name: "test topLN",
+			fields: fields{
+				fileName:         filepath.Join(testdata, "test3.txt"),
+				topLN:            10,
+				lastSearchLN:     9,
+				sectionHeaderNum: 3,
+			},
+			want: 10,
+		},
+		{
+			name: "test topLN2",
+			fields: fields{
+				fileName:         filepath.Join(testdata, "test3.txt"),
+				topLN:            10,
+				lastSearchLN:     14,
+				sectionHeaderNum: 3,
+			},
+			want: 10,
+		},
+		{
+			name: "test topLN3",
+			fields: fields{
+				fileName:         filepath.Join(testdata, "test3.txt"),
+				topLN:            10,
+				lastSearchLN:     8,
+				sectionHeaderNum: 3,
+			},
+			want: 10,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			root := rootFileReadHelper(t, tt.fields.fileName)
+			root.prepareScreen()
+			root.Doc.topLN = tt.fields.topLN
+			root.Doc.lastSearchLN = tt.fields.lastSearchLN
+			root.Doc.SectionHeaderNum = tt.fields.sectionHeaderNum
+			ctx := context.Background()
+			root.draw(ctx)
+			if got := root.startSearchLN(); got != tt.want {
+				t.Errorf("Root.startSearchLN() = %v, want %v", got, tt.want)
+			}
+		})
+	}
+}
+
+func TestRoot_searchMove(t *testing.T) {
+	tcellNewScreen = fakeScreen
+	defer func() {
+		tcellNewScreen = tcell.NewScreen
+	}()
+	type fields struct {
+		fileName string
+	}
+	type args struct {
+		forward  bool
+		lineNum  int
+		searcher Searcher
+		nonMatch bool
+	}
+	tests := []struct {
+		name   string
+		fields fields
+		args   args
+		want1  bool
+		want2  bool
+	}{
+		{
+			name: "test3 forward",
+			fields: fields{
+				fileName: filepath.Join(testdata, "test3.txt"),
+			},
+			args: args{
+				forward:  true,
+				lineNum:  0,
+				searcher: NewSearcher("10", regexpCompile("10", false), false, false),
+				nonMatch: false,
+			},
+			want1: true,
+			want2: true,
+		},
+		{
+			name: "test3 backForward",
+			fields: fields{
+				fileName: filepath.Join(testdata, "test3.txt"),
+			},
+			args: args{
+				forward:  false,
+				lineNum:  9000,
+				searcher: NewSearcher("1000", regexpCompile("1000", false), false, false),
+				nonMatch: false,
+			},
+			want1: true,
+			want2: true,
+		},
+		{
+			name: "test3 not found",
+			fields: fields{
+				fileName: filepath.Join(testdata, "test3.txt"),
+			},
+			args: args{
+				forward:  true,
+				lineNum:  0,
+				searcher: NewSearcher("test", regexpCompile("test", false), false, false),
+				nonMatch: false,
+			},
+			want1: false,
+			want2: false,
+		},
+		{
+			name: "test3 nonMatch",
+			fields: fields{
+				fileName: filepath.Join(testdata, "test3.txt"),
+			},
+			args: args{
+				forward:  true,
+				lineNum:  0,
+				searcher: NewSearcher("test", regexpCompile("test", false), false, false),
+				nonMatch: true,
+			},
+			want1: true,
+			want2: true,
+		},
+		{
+			name: "test3 backSearch nonMatch",
+			fields: fields{
+				fileName: filepath.Join(testdata, "test3.txt"),
+			},
+			args: args{
+				forward:  false,
+				lineNum:  1000,
+				searcher: NewSearcher("test", regexpCompile("test", false), false, false),
+				nonMatch: true,
+			},
+			want1: true,
+			want2: true,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			root := rootFileReadHelper(t, tt.fields.fileName)
+			root.prepareScreen()
+			ctx := context.Background()
+			root.everyUpdate(ctx)
+			root.draw(ctx)
+			root.Doc.nonMatch = tt.args.nonMatch
+			root.searcher = tt.args.searcher
+			if got := root.searchMove(ctx, tt.args.forward, tt.args.lineNum, tt.args.searcher); got != tt.want1 {
+				t.Errorf("Root.searchMove() = %v, want %v", got, tt.want1)
+			}
+			if eventF := root.Screen.HasPendingEvent(); eventF != tt.want2 {
+				t.Errorf("Root.searchMove() HasPendingEvent() = %v, want %v", eventF, tt.want2)
+			}
+		})
+	}
+}
+
+func TestRoot_incSearch(t *testing.T) {
+	tcellNewScreen = fakeScreen
+	defer func() {
+		tcellNewScreen = tcell.NewScreen
+	}()
+	type fields struct {
+		fileName string
+		topLN    int
+		word     string
+	}
+	type args struct {
+		forward bool
+		lineNum int
+	}
+	tests := []struct {
+		name   string
+		fields fields
+		args   args
+	}{
+		{
+			name: "test3 forward",
+
+			fields: fields{
+				fileName: filepath.Join(testdata, "test3.txt"),
+				word:     "10",
+			},
+			args: args{
+				forward: true,
+				lineNum: 0,
+			},
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			root := rootFileReadHelper(t, tt.fields.fileName)
+			root.Doc.topLN = tt.fields.topLN
+			root.input.value = tt.fields.word
+			root.prepareScreen()
+			ctx := context.Background()
+			root.incSearch(ctx, tt.args.forward, tt.args.lineNum)
+			root.returnStartPosition()
+			if got := root.Doc.topLN; got != tt.fields.topLN {
+				t.Errorf("Root.returnStartPosition() = %v, want %v", got, tt.fields.topLN)
+			}
+		})
+	}
+}
+
+func Test_cancelKeys(t *testing.T) {
+	type args struct {
+		c          *cbind.Configuration
+		cancelKeys []string
+		cancelApp  func(_ *tcell.EventKey) *tcell.EventKey
+	}
+	tests := []struct {
+		name    string
+		args    args
+		want    *cbind.Configuration
+		wantErr bool
+	}{
+		{
+			name: "test1",
+
+			args: args{
+				c:          cbind.NewConfiguration(),
+				cancelKeys: []string{"ctrl+c", "c"},
+				cancelApp:  nil,
+			},
+			wantErr: false,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			_, err := cancelKeys(tt.args.c, tt.args.cancelKeys, tt.args.cancelApp)
+			if (err != nil) != tt.wantErr {
+				t.Errorf("cancelKeys() error = %v, wantErr %v", err, tt.wantErr)
+				return
+			}
+		})
+	}
+}
